#ifndef _classes
#define _classes

#include <QGLWidget>
#include <QDialog>
#include <QModelIndex>
#include <QListView>
#include <QLabel>
#include <QLineEdit>
#include <QGroupBox>
#include <QSyntaxHighlighter>
#include <QFileDialog>

class QPushButton;
class QToolButton;
class QTabWidget;
class QSlider;
class QSpinBox;
class QCheckBox;
class QComboBox;
class QToolButton;
class QTextEdit;
class QTreeView;

//----------------------
class EmulWindow : public QGLWidget
{
	Q_OBJECT
	public:
		EmulWindow(QWidget*);
		void setpause(bool);
	public slots:
		void emulframe();
		void makemenu();
	signals:
		void onerror();
		void wannasetup();
		void wannadevelop();
	private:
		QMenu *menu;
		void paintmain();
		bool paused,mgrab;
		GLuint texture;
		GLuint displaylist;
	private slots:
		void actmenu(QAction*);
	protected:
		void resizeGL(int, int);
		void closeEvent(QCloseEvent*);
		void keyPressEvent(QKeyEvent*);
		void keyReleaseEvent(QKeyEvent*);
		void mousePressEvent(QMouseEvent*);
		void mouseReleaseEvent(QMouseEvent*);
		void mouseMoveEvent(QMouseEvent*);
};

class MyListView : public QListView {
	Q_OBJECT
public:
	void selectRow(int num) {
		setSelection(QRect(1,num,1,3),QItemSelectionModel::Select);
//		selectionModel()->select(QModelIndex(),QItemSelectionModel::Select);
	}
	int firstselectedrow() {
		QModelIndexList idxl = selectedIndexes();
		if (idxl.size()==0) return -1;
		return idxl[0].row();
	}
signals:
	void rowchanged(int,int);
protected slots:
	void currentChanged(QModelIndex cur, QModelIndex prev) {
		emit rowchanged(cur.row(),prev.row());
		QListView::currentChanged(cur,prev);
	}
};

class SetupWin : public QDialog
{
	Q_OBJECT
public:
	SetupWin(QWidget*);
public slots:
	void gosetup();
	void updateall();
	void setrsetcombox(int);
	void updateromset(int);
	void okexit();
	void opentape();void writetape();void ejecttape();void deletetapeblock();void tapeblockup();void tapeblockdn();
	void opendiska();void opendiskb();void opendiskc();void opendiskd();
	void savediska();void savediskb();void savediskc();void savediskd();
	void ejctdiska();void ejctdiskb();void ejctdiskc();void ejctdiskd();
	void choosesshfolder();
	void applysettings();
	void setcurtapeblock();
	void selsjapath();
private:
	QPushButton *okbut,*canbut,*applbut;
	QToolButton *seltapbut,*savtapbut,*clrtapbut;
	QPushButton *deltblock,*tblockup,*tblockdn;
	QToolButton *loadisk[4],*savdisk[4],*ejcdisk[4],*setsshfld,*sjasel;
	QSlider *sblensld,*beepsld,*tapesld,*aysld,*brdsld;
	QLabel *sblenshow,*beepshow,*tapeshow,*ayshow,*brdshow;
	QCheckBox *snden,*bdien,*dskten,*dskprt[4],*dsk80[4],*dskss[4],*dblsize,*fscrn,*sshotbrd;
	QCheckBox *resetmach,*insndchk;
	QTabWidget *tabz;
	QLineEdit *tapname,*tapblock,*diskname[4],*sshotfld;
	QLineEdit *sjaline;
	QComboBox *sndrate,*sshotformbox,*machcombox,*rsetcombox,*sndsysbox;
	QSpinBox *screpcount,*screptime;
	QListView *listrom;
	MyListView *listtape;
	QStringList flist;
	QDialog *umendial;
		QLineEdit *umdname,*umdpath;
		QToolButton *umdsel;
		QPushButton *umdok,*umdcan;
	QListView *menlist;
	QToolButton *addmen,*delmen;
	int editrow;
	void formumlist();

	void reject();
	void opendisk(unsigned char);
	void settapelist();
	uint temptapeblock;
private slots:
	void addnewumen();
	void editumen(QModelIndex);
	void applyaddumen();
	void selumdpath();
	void delusermen();
};

class ProjectFile {
public:
	QString name;
	QString text;
	bool changed;
};

class MHighlight : public QSyntaxHighlighter {
public: MHighlight(QTextEdit *prnt):QSyntaxHighlighter(prnt) {}
protected:
	void highlightBlock(const QString&);
};

class DevelWin : public QDialog {
	Q_OBJECT
public:	DevelWin(QWidget*);
private slots:
	void makenewproject();
	void saveproject();
	void addfiletoproject();
	void trueaddfiletoproject(QModelIndex);
	void delfilefromproject();
	void compile();
	void recalltext(int,int);
	void addnewfiletoproject();
private:
	void reject();
	void saveprojectfile();
	void getproject();
	void updateprojectlist();
	void storecurrentfile();
	QLineEdit *inpfname;
	QToolButton *newprj,*addfile,*delfile,*newfile;
	QToolButton *savbut,*compbut;
	QTextEdit *editor;
	MyListView *tree;
	QString basepath;		//	путь к папке с текущим проектом
	QList<ProjectFile> project;	//	список файлов проекта
	QTreeView *filetree;
	int currentfile;
	QTextEdit *cons;
	MHighlight *hlight;
};

struct MFResult {
	QString selfile;
	QString selfilt;
	int fidx;
};

class MFDialog : public QFileDialog {
	public:
		MFResult open(QWidget*,QString,QString,QStringList);
		MFResult save(QWidget*,QString,QString,QStringList);
		QDir lastdir;
	private:
		MFResult execute(QWidget*,QString,QString,QStringList);
};

#endif
