// STANDARD (FE, 7FFD)

void out_fe(unsigned char value) {
	brdcol=value&0x07;
	beeplevel=(value&0x10);
	tape.outsig=(value&0x08);
}

unsigned char in_fe(unsigned int portmap) {
	unsigned char res;
	res=getkeymap(portmap);		// input keyboard
	res=res|(tape.signal?0x40:0);
	return res;
}

void out_7ffd(unsigned char val) {
	if (block7ffd) return;
	mem.prt0 = val;
	mem.curram = (mem.curram & 0xf8) | (val & 0x07);
	curscrn = val & 0x08;
	block7ffd = val & 0x20;
}

//

uchar standardin(int port) {
	uchar res;
	res=0xff;
	if ((port&0xff)==0xfe) res=in_fe((port&0xff00)>>8);
	return res;
}

void standardout(int port, uchar value) {
	if ((port&0xff)==0xfe) out_fe(value);
}

// SIMPLE ZX48

int zx48_getport(int port) {
	port=bdi.getport(port);
	if (!bdi.pcatch) {
		if ((port&0x01)==0) port=(port&0xff00)|0xfe;
	}
	return port;
}

void zx48_setrom() {mem.currom = bdi.active?3:1;}

void zx48_outport(int port, uchar value) {
	standardout(port,value);
}

uchar zx48_inport(int port) {
	uchar res;
	res=standardin(port);
	return res;
}

// ZX128

int zx128_getport(int port) {
	port=bdi.getport(port);
	if (!bdi.pcatch) {
		if ((port&0x8002)==0x0000) port=0x7ffd;
		if ((port&0xc002)==0x8000) port=0xbffd;
		if ((port&0xc002)==0xc000) port=0xfffd;
		if ((port&0x05a1)==0x0081) port=0xfadf;
		if ((port&0x05a1)==0x0181) port=0xfbdf;
		if ((port&0x05a1)==0x0581) port=0xffdf;
	}
	return port;
}

void zx128_setrom() {mem.currom = (bdi.active)?3:((mem.prt0 & 0x10)>>4);}

void zx128_outport(int port, uchar value) {
	standardout(port,value);
	switch (port) {
		case 0x7ffd: out_7ffd(value); zx128_setrom(); break;
		case 0xfffd: aycurreg=value; break;			// установка регистра AY
		case 0xbffd: aysetreg(aycurreg,value); break;		// запись в регистр AY
	}
}

uchar zx128_inport(int port) {
	uchar res;
	res=standardin(port);
	switch (port) {
		case 0xfffd: if (aycurreg<14) {res=ayreg[aycurreg];} else {
				if ((ayreg[7]&0x40) && (aycurreg==14)) res=ayreg[14];
				if ((ayreg[7]&0x80) && (aycurreg==15)) res=ayreg[15];
			}
			break;
		case 0xfbdf: res = mouse.xpos; break;
		case 0xffdf: res = mouse.ypos; break;
		case 0xfadf: res = mouse.buttons; break;
	}
	return res;
}

// SCORPION

int scrp_getport(int port) {
	port=bdi.getport(port);
	if (!bdi.pcatch) {
		switch (port&0x23) {			// SCORPION: A0,A1,A5
			case 0x01: port=0xdd; break;			// printer
			case 0x03:			// mouse
				switch (port&0x0500) {
					case 0x0000: port=0xfadf; break;
					case 0x0100: port=0xfbdf; break;
					case 0x0500: port=0xffdf; break;
				}
				break;
			case 0x21:
				switch (port&0xc000) {
					case 0x0000: port=0x1ffd; break;	// scorpion
					case 0x4000: port=0x7ffd; break;	// pentagon
					case 0x8000: port=0xbffd; break;	// AY 1
					case 0xc000: port=0xfffd; break;	// AY 2
				}
				break;
			case 0x22: port=(port&0xff00)|0xfe; break;	// FE
			case 0x23: port=0xff; break;			// FF
		}
	}
	return port;
}

void scrp_setrom() {
	if (mem.prt1 & 0x01) {mem.currom = 0xff;}
		else {mem.currom = ((mem.prt0 & 0x10) && !(mem.prt1 & 0x02)) | ((bdi.active || (mem.prt1 & 0x02))<<1);}
}

void scrp_outport(int port,unsigned char val) {
	standardout(port,val);
	switch (port) {
		case 0x7ffd:	out_7ffd(val); scrp_setrom(); break;
		case 0x1ffd:	mem.curram = (mem.prt0 & 7) | ((val & 0x10)>>1) | ((val&0xc0)>>2);
				mem.prt1 = val; scrp_setrom();
				break;
		case 0xfffd:	aycurreg=val; break;				// установка регистра AY
		case 0xbffd:	aysetreg(aycurreg,val); break;			// запись в регистр AY
	}
}

unsigned char scrp_inport(int port) {
	uchar res=standardin(port);
	switch (port) {
		case 0xff: if (!hbrd && !vbrd) {res=atrbyte;} break;
		case 0xfbdf: res = mouse.xpos; break;
		case 0xffdf: res = mouse.ypos; break;
		case 0xfadf: res = mouse.buttons; break;
		case 0xfffd: if (aycurreg<14) {res=ayreg[aycurreg];} else {
			if ((ayreg[7]&0x40) && (aycurreg==14)) res=ayreg[14];
			if ((ayreg[7]&0x80) && (aycurreg==15)) res=ayreg[15];
		}
		break;
	}
	return res;
}

// DEFINE

uchar getromsetindex(std::string nam) {
	uchar i;
	for (i=0; i<romslist.size(); i++) {
		if (nam == romslist[i].name) return i;
	}
	return 255;
}

void appendmachine(std::string mname, int (*gpf)(int),void (*opf)(int,uchar),uchar (*ipf)(int), void(*srf)(), uchar rrom) {
	MachineType newm;
	newm.name = mname;
	newm.getport = gpf;
	newm.out = opf;
	newm.in = ipf;
	newm.setrom = srf;
	newm.resrom = rrom;
	machlist.push_back(newm);
}
