#include <QtGui>
#include <QtOpenGL>


// sound

#ifndef WIN32
// alsa
	#include <alsa/asoundlib.h>
//oss
	#include <sys/ioctl.h>
	#include <sys/soundcard.h>
	#include <fcntl.h>
#else
	#include <mmsystem.h>
	#include <direct.h>
#endif

// system
#include <iostream>
#include <sys/time.h>
#include <string>
#include <fstream>
#include <sstream>
#include <vector>
#include <exception>
// all

#include "vars.h"
#include "classes.h"
#include "debugwin.h"

EmulWindow *emulwin;
MFDialog *fdial;
DebugWin *dbg;

#include "sound.c"
#include "common.c"
#include "clocks.c"

#include "debuger.c"

#include "machines.c"
#include "z80.c"

#include "vg93.c"

#include "snapshot.c"
#include "opensave.c"
#include "floppy.c"
#include "tape.c"

#include "emulwin.c"

#include "develop.c"
#include "setupfile.c"
#include "setupwin.c"

int main(int ac, char *av[]) {
	QApplication app(ac,av);
	int i;

// если надо, создаем директорию установок
	addsoundsys("NULL",&null_open,&null_play,&null_close);
#ifndef WIN32
	mkdir(workdir.c_str(),0777);
	mkdir(romdir.c_str(),0777);
	addsoundsys("OSS",&oss_open,&oss_play,&oss_close);
	addsoundsys("ALSA",&alsa_open,&alsa_play,&alsa_close);
	cursndsys=2;
#else
	mkdir(workdir.c_str());
	mkdir(romdir.c_str());
	addsoundsys("WaveOut",&wave_open,&wave_play,&wave_close);
	cursndsys=1;
#endif
	emptyrs.name = "Error";
	for (i=0;i<8;i++) {emptyrs.path.push_back(""); emptyrs.part[i]=0;}
	appendmachine("ZX48K",&zx48_getport,&zx48_outport,&zx48_inport,&zx48_setrom,1);
	appendmachine("ZX128K",&zx128_getport,&zx128_outport,&zx128_inport,&zx128_setrom,0);
	appendmachine("Scorpion",&scrp_getport,&scrp_outport,&scrp_inport,&scrp_setrom,0);
	curmachine=2;

// init floppys
	for(i=0;i<4;i++) {
		flop[i].speed=42000; flop[i].count=0;
		flop[i].trk80=true; flop[i].dblsid=true;
		flop[i].id=i;
	}
	tape.cansave = true;
	vg93.fptr = &flop[0];
// toolz
	std::ifstream sjf("/usr/bin/sjasmplus");
	if (sjf.good()) {sjapath="/usr/bin/sjasmplus"; sjf.close();}
	loadsettings();
// init tables
	initladrz();
// init sound
	cursoundsys = &soundsyslist[cursndsys];
	defsoundparamz();
	if (!cursoundsys->open()) {
		QMessageBox mbx; mbx.setIcon(QMessageBox::Critical);
		mbx.setText("Open sound device failed");
		mbx.setInformativeText("Reset to NULL audio output");
		mbx.exec();
		cursndsys=0;
		cursoundsys = &soundsyslist[0];
	}
// тачанка
	machine = &machlist[curmachine];
	loadmachineromset();
// input paramz
	if (ac>1) {for (i=1;i<ac;i++) loadsomefile(std::string(av[i]));}
	mouse.buttons = 0xff;
	mouse.xpos = mouse.ypos = 0;
// create windows
	emulwin = new EmulWindow;
	SetupWin setupa(emulwin);
	DevelWin devwin;
	fdial = new MFDialog;
		fdial->lastdir = QDir::home();
	dbg = new DebugWin(emulwin);

	emulwin->show();
	reset();

	QTimer timer;
	QObject::connect(&timer,SIGNAL(timeout()),emulwin,SLOT(emulframe()));
	QObject::connect(emulwin,SIGNAL(wannadevelop()),&devwin,SLOT(show()));
	QObject::connect(emulwin,SIGNAL(wannasetup()),&setupa,SLOT(gosetup()));
	timer.start(20);
	
	try {
		app.exec();
		cursoundsys->close();
		return 0;
	}
	catch (std::exception exc) {
		printf("%s\n",exc.what());
	}
}
