void SetupWin::writetape() {
	if (tape.data.size()==0) return;	// ленты нет
	savetape(tape.path);
	tapname->setText(tape.path.c_str());
}

void SetupWin::opentape() {
	QString fnam = QFileDialog::getOpenFileName(this,"Open tape","","Tape files (*.tap *.tzx)");
	if (fnam!="") {
		tapname->setText(fnam);
		tape.path = std::string(fnam.toUtf8().data());
		loadtape(tape.path);
	}
	settapelist();
}

void SetupWin::ejecttape() {
	tapname->clear();
	tape.eject();
	settapelist();
}

void SetupWin::tapeblockup() {
	QModelIndex idx = listtape->currentIndex();
	int row = idx.row();
	if (row < 1) return;
	tape.swapblocks(row,row-1);
	settapelist();
	listtape->selectRow(row-1);
}

void SetupWin::tapeblockdn() {
	QModelIndex idx = listtape->currentIndex();
	int row = idx.row();
	if (row == -1) return;
	if (row >= (int)tape.data.size()) return;
	tape.swapblocks(row,row+1);
	settapelist();
	listtape->selectRow(row+1);
}

void SetupWin::deletetapeblock() {
	int row = listtape->currentIndex().row();
	if (row != -1) {
		tape.data.erase(tape.data.begin()+row);
		settapelist();
	}
}

void SetupWin::setcurtapeblock() {
	QAbstractItemModel *model = listtape->model();
	QModelIndex idx = listtape->currentIndex();
	tapblock->setText(model->data(idx).toString());
	temptapeblock = idx.row();
}

void SetupWin::settapelist() {
	QStringListModel *model = new QStringListModel;
	QStringList list;
	QString element;
	uint i,j;
	int dlen,dpos,timez,tmin,tsec;
	for (i=0;i<tape.data.size();i++) {
		element = QString::number(i+0x10000,16).right(4).append("  ");
		dpos=tape.data[i].datapos;
		dlen=((tape.data[i].data.size()-dpos)>>4) - 2;			// длина в байтах без типа и контрольного
		timez=0;
		for (j=0;j<tape.data[i].data.size();j++) {timez += tape.data[i].data[j];}
		timez = int(timez / 3584000.0);
		tmin = int(timez/60.0);
		tsec = timez - tmin*60;
		element.append(QString::number(tmin)).append(":").append(QString::number(tsec+100).right(2)).append("\t");
		element.append(QString::number(dlen)).append(" bytes\t");
		if (tape.getbyte(i,dpos) == 0) {
			if (tape.getbyte(i,dpos+16)==0) {element.append("Prog:");} else {element.append("Code:");}
			element.append(QString((const char*)&tape.getdata(i,dpos+32,10)[0]).left(10));
		}
		list<<element;
	}
	model->setStringList(list);
	listtape->setModel(model);
	if ((list.size()!=0) and (tape.block<tape.data.size())) {tapblock->setText(list[tape.block]);} else {tapblock->clear();}
}

// Tape funcs ----------------------------

void Tape::eject() {
	play=rec=false;
	block=pos=0;
	path="";
	data.clear();
	cansave = true;
}

void Tape::tick() {
	if (ticks>0) {ticks--; return;}
	if (rec) {
		rectick();
	} else {
		signal=!signal;
		if (play) {
			if (pos < data[block].data.size()) {
				ticks = data[block].data[pos];
				pos++;
			} else {
				block++;
				pos=0;
				if (block>=data.size()) play=false;
			}
		} else {ticks = 320*448*25;}
	}
}

void Tape::rectick() {
	int lastbyte = blockptr.data.size()-1;
	if (toutold == outsig) {blockptr.data[lastbyte]++;}
		else {toutold = outsig; blockptr.data.push_back(0);}
	if ((blockptr.data.back() > 320*448*2) && (lastbyte!=0)) {	// сигнал более чем в 2 прерывания считается концом блока
		blockptr.data.pop_back();				// это пауза в 2 прерывания
//		blockptr.data[0]=320*448*50;				// 1я пауза примерно в секунду
		uint i,j;			// нормируем сигнал. разброс амплитуды +-10 тиков @ 7MHz
		bool same;
		std::vector<uint> rval;
		for (i=1;i<blockptr.data.size();i++) {
			same = false;
			for(j=0;j<rval.size();j++) {
				if ((uint)(blockptr.data[i] - rval[j] + 10) < 20) {blockptr.data[i] = rval[j]; same=true;}
			}
			if (!same) rval.push_back(blockptr.data[i]);
		}
//		printf("%i\n",rval.size());
		if (rval.size() == 6) {
			blockptr.plen = rval[0];
			blockptr.s1len = rval[1];
			blockptr.s2len = rval[2];
			if (rval[4]>rval[3]) {
				blockptr.len0 = rval[3];
				blockptr.len1 = rval[4];
			} else {
				blockptr.len0 = rval[4];
				blockptr.len1 = rval[3];
			}
			blockptr.datapos=-1;
			i=1;
			while (blockptr.data[i]!=rval[2]) i++;
			blockptr.datapos = i+1;
			data.push_back(blockptr);
		}
		blockptr.data.clear();
		blockptr.data.push_back(0);
	}
}

void Tape::swapblocks(int r1,int r2) {
	TapeBlock tb1 = data[r1];
	data[r1] = data[r2];
	data[r2] = tb1;
}

uchar Tape::getbyte(int blk, int pos) {
	uchar i,res=0;
	for (i=0;i<8;i++) {
		if (data[blk].data[pos]==data[blk].len1) {res |= tmasks[7-i];}
		pos += 2;
	}
	return res;
}

std::vector<uchar> Tape::getdata(int blk, int pos, int len) {
	std::vector<uchar> res;
	if (pos==-1) pos=data[blk].datapos;		// с начала
	if (len==-1) len=int((data[blk].data.size()-pos)>>4);	// до конца (в байтах)
	while (len!=0) {
		res.push_back(getbyte(blk,pos));
		pos += 16;
		len--;
	}
	return res;
}

// переконвертить len байт из *file в сигналы мофона. длины сигналов в slens, isnew=true если это новый блок
TapeBlock Tape::parse(std::ifstream *file, uint len, std::vector<int> slens) {
	TapeBlock newb;
	uint i;
	uint8_t j;
	tmp = file->peek();
	newb.plen=slens[0];
	newb.s1len=slens[1];
	newb.s2len=slens[2];
	newb.len0=slens[3];
	newb.len1=slens[4];
	newb.pdur = (slens[6]==-1)?((tmp&0x80)?3223:8063):slens[6];
	newb.data.clear();
//	newb.data.push_back(448*320*((tmp&0x80)?25:50));		// 1 sec before header, .5 before data
	if (newb.pdur!=0) {
		for (i=0;i<newb.pdur;i++) newb.data.push_back(newb.plen);
	}
	if (newb.s1len!=0) newb.data.push_back(newb.s1len);
	if (newb.s2len!=0) newb.data.push_back(newb.s2len);
	newb.datapos = newb.data.size();
	for (i=0;i<len;i++) {
		file->read((char*)&tmp,1);
		for (j=0;j<8;j++) {
			if (tmp & 0x80) {newb.data.push_back(newb.len1);newb.data.push_back(newb.len1);}
				else {newb.data.push_back(newb.len0);newb.data.push_back(newb.len0);}
			tmp = (tmp<<1);
		}
	}
	if (slens[5]!=0) newb.data.push_back(slens[5]);
	return newb;
}

uint getlen(std::ifstream *file,uchar n) {
	uint len;
	uchar tm;
	file->read((char*)&tm,1); len = tm;
	if (n > 1) {file->read((char*)&tm,1); len += (tm<<8);}
	if (n > 2) {file->read((char*)&tm,1); len += (tm<<16);}
	if (n > 3) {file->read((char*)&tm,1); len += (tm<<24);}
	return len;
}

void Tape::load(std::string sfnam,uchar type) {
	QMessageBox mbx; mbx.setIcon(QMessageBox::Critical); mbx.setWindowTitle("Error");
	std::ifstream file(sfnam.c_str(),std::ios::binary);
	if (!file.good()) {
		mbx.setText("<b>Can't open file</b>");
		mbx.setInformativeText(sfnam.c_str());
		mbx.exec();
		return;
	}
	uint len,paulen,loopc=0;
	std::streampos loopos = 0;
	int sd[] = {2168,667,735,855,1710,954,-1};
	std::vector<int> slens = std::vector<int>(sd,sd + sizeof(sd) / sizeof(int));
	std::vector<int> alens = slens;
	uchar *buf = new uchar[256];
	bool err = true;
	TapeBlock newb;
	eject();
	switch (type) {
		case TYPE_TAP:
					// длины сигналов (pilot,s1,s2,0,1,s3,pilotsize (-1==auto))
			while (!file.eof()) {
				len = getlen(&file,2);
				if (!file.eof()) {
					newb = parse(&file,len,slens);
					newb.data.push_back(448*320*((newb.pdur==8063)?25:50));
					data.push_back(newb);
				}
			}
			path=sfnam;
			break;
		case TYPE_TZX:
//			printf("Loading TZX in under construction\n"); break;
			file.read((char*)buf,10);
			if ((std::string((char*)buf,7) != "ZXTape!") || (buf[7]!=0x1a)) {
				mbx.setText("<b>Wrong TZX file</b>");
				mbx.setInformativeText("Incorrect signature");
				mbx.exec();
				break;
			}
			file.read((char*)&tmp,1);
			do {
				switch (tmp) {
					case 0x10:
						paulen = getlen(&file,2);
						len = getlen(&file,2);
						newb = parse(&file,len,slens);
						newb.data.push_back((hticks*vlines/20)*paulen);
						data.push_back(newb);
						break;
					case 0x11:
						alens[0] = getlen(&file,2);	// pilot
						alens[1] = getlen(&file,2);	// sync1
						alens[2] = getlen(&file,2);	// sync2
						alens[3] = getlen(&file,2);	// 0
						alens[4] = getlen(&file,2);	// 1
						alens[6] = getlen(&file,2);	// pilot pulses
						file.get();
						paulen = getlen(&file,2);
						len = getlen(&file,3);
						newb = parse(&file,len,alens);
						newb.data.push_back((hticks*vlines/20)*paulen);
						data.push_back(newb);
						cansave = false;
						break;
					case 0x12:
						if (data.size()==0) {newb.data.clear(); data.push_back(newb);}
						paulen = getlen(&file,2);
						len = getlen(&file,2);
						while (len>0) {data.back().data.push_back(paulen); len--;}
						cansave = false;
						break;
					case 0x13:
						if (data.size()==0) {newb.data.clear(); data.push_back(newb);}
						len = file.get();
						while (len>0) {data.back().data.push_back(file.get()); len--;}
						cansave =false;
						break;
					case 0x14:
						alens[1]=alens[2]=alens[6]=0;	// no pilot, sync1, sync2
						alens[3]=getlen(&file,2);
						alens[4]=getlen(&file,2);
						file.get();
						paulen = getlen(&file,2);
						len = getlen(&file,3);
						newb = parse(&file,len,alens);
						newb.data.push_back((hticks*vlines/20)*paulen);
						data.push_back(newb);
						cansave=false;
						break;
					case 0x15:
						printf("Block 0x15 skipped\n");
						file.seekg(5,std::ios::cur);
						len = getlen(&file,3);
						file.seekg(len,std::ios::cur);
						break;
					case 0x18:
						printf("Block 0x18 skipped\n");
						len = getlen(&file,4);
						file.seekg(len,std::ios::cur);
						break;
					case 0x19:
						printf("Block 0x19 skipped\n");
						len = getlen(&file,4);
						file.seekg(len,std::ios::cur);
						break;
					case 0x20:
						if (data.size()==0) {newb.data.clear(); data.push_back(newb);}
						len = getlen(&file,2);
						data.back().data.push_back(len);
						break;
					case 0x21:
						printf("Block 0x21 skipped\n");
						len = file.get();
						file.seekg(len,std::ios::cur);
						break;
					case 0x22:
						break;
					case 0x23:
						printf("Block 0x23 skipped\n");
						file.seekg(2,std::ios::cur);
						break;
					case 0x24:
						loopc = getlen(&file,2);
						loopos = file.tellg();
						break;
					case 0x25:
						if (loopc>0) {loopc--; if (loopc!=0) file.seekg(loopos,std::ios::beg);}
						break;
					case 0x26:
						printf("Block 0x26 skipped\n");
						len = getlen(&file,2);
						file.seekg(len<<1,std::ios::cur);
						break;
					case 0x27:
						break;
					case 0x28:
						printf("Block 0x28 skipped\n");
						len = getlen(&file,2);
						file.seekg(len<<1,std::ios::cur);
						break;
					case 0x2a:
						printf("Block 0x2a skipped\n");
						file.seekg(4,std::ios::cur);
						break;
					case 0x2b:
						printf("Block 0x2b skipped\n");
						file.seekg(5,std::ios::cur);
						break;
					case 0x30:
						len = file.get();
						file.seekg(len,std::ios::cur);
						break;
					case 0x31:
						file.get();
						len = file.get();
						file.seekg(len,std::ios::cur);
						break;
					case 0x32:
						len = getlen(&file,2);
						file.seekg(len,std::ios::cur);
						break;
					case 0x33:
						len = getlen(&file,1);
						file.seekg(len*3,std::ios::cur);
						break;
					case 0x34:
						file.seekg(4,std::ios::cur);
						len = getlen(&file,4);
						file.seekg(len,std::ios::cur);
						break;
					case 0x5a:
						file.seekg(9,std::ios::cur);
						break;
					default:
						mbx.setText("Unknown TZX block");
						mbx.setDetailedText(QString("block type\t0x").append(QString::number(tmp,16)));
						mbx.exec();
						err = false;
						break;
				}
				file.read((char*)&tmp,1);
			} while (!file.eof() && err);
	}
}

void Tape::save(std::string fname, uchar type) {
	std::ofstream file(fname.c_str(),std::ios::binary);
	std::vector<uchar> buf;
	uchar lenh,lenl;
	uint8_t i;
	switch (type) {
		case TYPE_TAP:
			for (i=0;i<data.size();i++) {
				buf = getdata(i,-1,-1);
				lenl = buf.size()&0xff;
				lenh = ((buf.size()&0xff00)>>8);
				file.write((char*)&lenl,1);
				file.write((char*)&lenh,1);
				file.write((char*)&buf[0],buf.size());
			}
			break;
	}
	file.close();
}
