struct Memory {
	uchar ram[64][16384];
	uchar rom[8][16384];
	uchar curram,currom;
	uchar prt0;		// 7ffd value
	uchar prt1;		// extend port value
	uchar rd(ushort);
	void wr(ushort,uchar);
};
Memory mem;

uchar reg[14];	// b,c,d,e,h,l,f,a,hsp,lsp,hx,lx,hy,ly
uchar alt[8];	// b,c,d,e,h,l,f,a
ushort pc;
uchar ireg,rreg,imode;
bool iff1,iff2;
uchar delta;

uchar screen[512*400*4];

long acount=0,intcount=0,inintcount=0;
uchar *scrptr;
int hticks=448,hbrdlen=128,hsynclen=64;
int vlines=320,vbrdlen=80,vsynclen=32;
int intlen=64;
uchar gfxmode=0;	// 1: 16col alco
int hlcut=hsynclen,hrcut=hticks;
int vucut=vsynclen,vdcut=vlines;
int xscrsize=hrcut-hlcut;
int yscrsize=vdcut-vucut;
float zoom=1.0;
bool hbrd,vbrd,hsync,vsync,intupt,flash;
int dlinepos = 3*xscrsize*(yscrsize-1);
int twolines = 6*xscrsize;
int hcount=0,vcount=0;
int scradr,atradr;
uint tcount=0;
uchar scrbyte,atrbyte,ink,pap,brdcol=2,fclock;
bool curscrn=false;			// false:5	true:7 screen
bool doublesize=true;
bool fullscrn=false;

ushort ladrz[0x1800][2];	// 0:scradr	1:atradr
ushort iacount=0;

uchar bordersize=50;	// % бордюра
int scrposx,scrposy;
bool needsavescreenshot=false;
int screenshotnum=0;
bool borderinscreenshot=true;
int sshotrpttime=50, sshotrptcount=60;
int sshotbcktime=0, sshotbckcount=0;
int rdr_leftpos,rdr_sizex;
int rdr_uppos,rdr_sizey;

bool block7ffd=false;
bool ixpref,iypref,edpref,cbpref,nextei=false;
bool error,executed=false;
int tempfps;

const uchar tmasks[8]={0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
const uchar imasks[8]={0xfe,0xfd,0xfb,0xf7,0xef,0xdf,0xbf,0x7f};

const uchar c0=0x00,c1=0xa0,c2=0xff;
int palete[3][16]=	{{c0,c0,c1,c1,c0,c0,c1,c1,c0,c0,c2,c2,c0,c0,c2,c2},\
			 {c0,c0,c0,c0,c1,c1,c1,c1,c0,c0,c0,c0,c2,c2,c2,c2},\
			 {c0,c1,c0,c1,c0,c1,c0,c1,c0,c2,c0,c2,c0,c2,c0,c2}};

//QDir lastdirectory=QDir::homePath();					// папка для опенсейвов

std::string screenshotformat="png";
#ifndef WIN32
	std::string screenshotdir=std::string(getenv("HOME"));
	std::string workdir = std::string(getenv("HOME"))+"/.samstyle/samulator";
#else
	std::string screenshotdir=std::string("./");
	std::string workdir = std::string("./config");
#endif
std::string romdir = std::string(workdir)+"/roms";
std::string optpath = std::string(workdir)+"/samulator.conf";
std::string sjapath;

struct MenuEntry {
	std::string name;
	std::string path;
};
std::vector<MenuEntry> usermenu,tumlist;

// machine

struct RomSet {
	std::string name;
	std::vector<std::string> path;
	uchar part[8];
};
RomSet emptyrs;

struct MachineType {
	std::string name;
	RomSet *romset;
	int (*getport)(int);
	void (*out)(int,unsigned char);
	uchar (*in)(int);
	void (*setrom)();
	uchar resrom;
};

MachineType *machine;// = new MachineType;
std::vector<MachineType> machlist;
std::vector<RomSet> romslist;
uchar curmachine;
bool resetafterchange=true;

// IO sys

struct Mouse {
	uchar xpos;
	uchar ypos;
	uchar buttons;	// b0=LMB; b1=RMB;
};
Mouse mouse;

// tape

#define	TYPE_TAP	0
#define	TYPE_TZX	1

struct TapeBlock {
	uint plen,s1len,s2len,len0,len1;				// длины сигналов
	uint pdur;						// сколько смен на пилота
	int datapos;						// номер первого сигнала данных
	std::vector<uint> data;					// длины сигналов по всей длине блока
};

struct Tape {
	std::string path;
	uint block;		// tape block
	uint pos;		// tap position (номер сигнала);
	bool play;
	bool rec;
	bool signal;
	bool toutold;
	bool outsig;
	bool cansave;		// можно ли ленту сохранить в tap (загрузка tap или tzx с блоком 0x10)
	int ticks;		// счетчик тиков при включеной ленте
	TapeBlock blockptr;	// временный блок. сюда ведется запись
	std::vector<TapeBlock> data;
	uchar getbyte(int,int);
	std::vector<uchar> getdata(int,int,int);
	void eject();
	void tick();
	void rectick();
	void swapblocks(int,int);
	TapeBlock parse(std::ifstream*,uint,std::vector<int>);
	void load(std::string,uchar);
	void save(std::string,uchar);
} tape;

// sound

struct SoundSys {
	std::string name;
	bool (*open)();
	void (*play)();
	void (*close)();
};

uchar sndbufer[2048];
bool soundenabled=true;
bool inactivesnd=true;
std::vector<SoundSys> soundsyslist;
SoundSys *cursoundsys;
uchar cursndsys;
int sndtick=0;

#ifndef WIN32
// alsa
	static char *device = (char*)"default";
	snd_output_t *output = NULL;
	snd_pcm_t *ahandle;
	snd_pcm_sframes_t frames;
// oss
	int audio;
	int sndformat=AFMT_U8;
#else
	HWAVEOUT hwaveout;
	WAVEFORMATEX wfx;
	WAVEHDR whdr;
#endif

int rate;
int sndfreqs[3]={44100,22050,11025};	// список поддерживаемых частот
uchar cursndfreq=0;		// номер текущей частоты
int chanels=1;
float sndbuflen=20;		// длина аудио буфера в мс
int bufsize;
uchar beepvolume=16,tapevolume=16,ayvolume=16;
int ticksatbyte,sndlimit=0;	// тиков на байт буфера звука и следующая граница тикалки для буфера (наращивается)
//int anote,bnote,cnote;
//int envelope;
int ayfreq=1774400;			// частота музпроца (1750000, 1774400)
uchar ayreg[16];		// регистры музпроца
float aycoe;
int sndbufpos;				// позиция в буфере
bool beeplevel;
bool alevel,blevel,clevel,nlevel;		// уровни сигналов звука и шума (1:0)
uchar avol,bvol,cvol,level;

//int sndtick=0;		// вообще тикалка звука (счетчик тактов)
uint aticks=0,alimit=0;		// тиков на полупериод А и следующая граница
uint bticks=0,blimit=0;		// то же для B
uint cticks=0,climit=0;		// то же для C
uint nticks=0,nlimit=0;		// то же для шума
uint eticks=0,elimit=0;		// то же для огибающей
int noisevalue=0xffff;			// NOIZZZE generation value
uchar aycurreg=0;
					// 0,1:A note	2,3:B note	4,5:C note
					// 6:noise frq	7:sound.reg	8,9,10:A,B,C volumes
					// 11,12: evnl	13:evnl.form	14,15:?

// формы огибающих. 253 - к началу. 255 - шаг на 1 назад
uchar envforms[16][33]={\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,255},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,255},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,255},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,255},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,255},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,255},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,255},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,255},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,253},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,255},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,253},\
	{15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,15,255},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,253},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,255},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,253},\
	{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,255}};
uchar envpos=0;
uchar curenv=0;

// keyboard

uchar keyboardmap[8]={0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
qint32 keyz[][5] = {
	{Qt::Key_1,4,1,0,0},{Qt::Key_2,4,2,0,0},{Qt::Key_3,4,4,0,0},{Qt::Key_4,4,8,0,0},{Qt::Key_5,4,16,0,0},
	{Qt::Key_6,3,16,0,0},{Qt::Key_7,3,8,0,0},{Qt::Key_8,3,4,0,0},{Qt::Key_9,3,2,0,0},{Qt::Key_0,3,1,0,0},
	{Qt::Key_Q,5,1,0,0},{Qt::Key_W,5,2,0,0},{Qt::Key_E,5,4,0,0},{Qt::Key_R,5,8,0,0},{Qt::Key_T,5,16,0,0},
	{Qt::Key_Y,2,16,0,0},{Qt::Key_U,2,8,0,0},{Qt::Key_I,2,4,0,0},{Qt::Key_O,2,2,0,0},{Qt::Key_P,2,1,0,0},
	{Qt::Key_A,6,1,0,0},{Qt::Key_S,6,2,0,0},{Qt::Key_D,6,4,0,0},{Qt::Key_F,6,8,0,0},{Qt::Key_G,6,16,0,0},
	{Qt::Key_H,1,16,0,0},{Qt::Key_J,1,8,0,0},{Qt::Key_K,1,4,0,0},{Qt::Key_L,1,2,0,0},{Qt::Key_Return,1,1,0,0},
	{Qt::Key_Shift,7,1,0,0},{Qt::Key_Z,7,2,0,0},{Qt::Key_X,7,4,0,0},{Qt::Key_C,7,8,0,0},{Qt::Key_V,7,16,0,0},
	{Qt::Key_B,0,16,0,0},{Qt::Key_N,0,8,0,0},{Qt::Key_M,0,4,0,0},{Qt::Key_Control,0,2,0,0},{Qt::Key_Space,0,1,0,0},
	
	{Qt::Key_Exclam,7,1,4,1},{Qt::Key_At,7,1,4,2},{Qt::Key_NumberSign,7,1,4,4},{Qt::Key_Dollar,7,1,4,8},{Qt::Key_Percent,7,1,4,16},
	{Qt::Key_AsciiCircum,7,1,3,16},{Qt::Key_Ampersand,7,1,3,8},{Qt::Key_Asterisk,7,1,3,4},{Qt::Key_ParenLeft,7,1,3,2},{Qt::Key_ParenRight,7,1,3,1},
	
	{Qt::Key_Tab,0,1,7,1},{Qt::Key_Left,4,16,7,1},{Qt::Key_Down,3,16,7,1},{Qt::Key_Up,3,8,7,1},{Qt::Key_Right,3,4,7,1},
	{Qt::Key_Minus,1,8,0,2},{Qt::Key_Equal,1,4,0,2},{Qt::Key_Backspace,3,1,7,1},
	{Qt::Key_Backslash,0,2,7,1},{Qt::Key_QuoteLeft,4,1,7,1},
	{Qt::Key_PageUp,7,1,4,4},{Qt::Key_PageDown,7,1,4,8},
	{Qt::Key_Home,0,2,5,1},{Qt::Key_End,0,2,5,2},{Qt::Key_Insert,0,2,5,4},
	{Qt::Key_BracketLeft,0,2,3,4},{Qt::Key_BracketRight,0,2,3,2},
	{Qt::Key_Semicolon,0,2,2,2},{Qt::Key_Apostrophe,0,2,2,1},
	{Qt::Key_Comma,0,2,0,8},{Qt::Key_Period,0,2,0,4},{Qt::Key_Slash,0,0,0,0},
	{0,0,0,0,0}
};
/*
uchar keys[256][2][2]={
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 0
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 5 
	{{4,1},{0,0}},{{4,2},{0,0}},{{4,4},{0,0}},{{4,8},{0,0}},{{4,16},{0,0}},		// 10: 1-5
	{{3,16},{0,0}},{{3,8},{0,0}},{{3,4},{0,0}},{{3,2},{0,0}},{{3,1},{0,0}},		// 15: 6-0
	{{0,2},{1,8}},{{0,2},{1,4}},{{7,1},{3,1}},{{7,1},{0,1}},			// 20: - + bcksp tab(brk)
	{{5,1},{0,0}},{{5,2},{0,0}},{{5,4},{0,0}},{{5,8},{0,0}},{{5,16},{0,0}},		// 24-28: q,w,e,r,t
	{{2,16},{0,0}},{{2,8},{0,0}},{{2,4},{0,0}},{{2,2},{0,0}},{{2,1},{0,0}},		// 29-33: y,u,i,o,p
	{{3,4},{0,2}},{{3,2},{0,2}},{{1,1},{0,0}},{{0,2},{0,0}},			// 34-37: [,],ent,ctrl
	{{6,1},{0,0}},{{6,2},{0,0}},{{6,4},{0,0}},{{6,8},{0,0}},{{6,16},{0,0}},		// 38-42: a,s,d,f,g
	{{1,16},{0,0}},{{1,8},{0,0}},{{1,4},{0,0}},{{1,2},{0,0}},{{0,2},{2,2}},		// 43-47: h,j,k,l,;
	{{2,1},{0,2}},{{0,2},{7,1}},{{7,1},{0,0}},					// 48-50: ',`,shift
	{{0,0},{0,0}},{{7,2},{0,0}},{{7,4},{0,0}},{{7,8},{0,0}},{{7,16},{0,0}},		// 51-55: \,z,x,c,v
	{{0,16},{0,0}},{{0,8},{0,0}},{{0,4},{0,0}},{{0,2},{0,8}},{{0,2},{0,4}},		// 56-60: b,n,m,,,.
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,1},{0,0}},		// 61-65:  , , , ,spc
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 66-70
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 71-75
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 76-80
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 81-85
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 86-90
	{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,0},{0,0}},		// 91-95
	{{0,0},{0,0}},{{0,2},{5,1}},{{7,1},{3,8}},{{7,1},{4,4}},{{7,1},{4,16}},		// 96-100 97:home(SS+Q) 98:up 99:pgup 100:lft
	{{0,0},{0,0}},{{7,1},{3,4}},{{0,2},{5,4}},{{7,1},{3,16}},{{7,1},{4,8}},		// 101-105 102:right 103:end(SS+E) 104:down 105:pgdn
	{{0,2},{5,2}},{{7,1},{3,2}},{{0,0},{0,0}},{{0,0},{0,0}},{{0,2},{5,1}},		//  106:ins(SS+W) 107:delete(CS+9) 110:home
	{{7,1},{3,8}},{{7,1},{4,4}},{{7,1},{4,16}},{{7,1},{3,4}},{{0,2},{5,4}},		// 111:up 112:pgup 113:lf 114:rg 115:end
	{{7,1},{3,16}},{{7,1},{4,8}},{{0,2},{5,2}},{{7,1},{3,2}}			// 116:dn 117:pgup 118:ins 119:del
	};
*/
// temp
uchar tmp,tmp2;
int tadr;
bool traceout=false;

// BDI

#define TRACKLEN 6250
#define INSTIME 1			// на будущее - время смены дискеты
#define TYPE_TRD 0
#define TYPE_SCL 1

int delays[4]={43000,84000,140000,224000};		// задержки ВГ93 при перемещении головки (6,12,20,32 ms)

struct BDI {
	bool enable;		// есть-нет
	bool active;		// активен дос
	bool pcatch;		// порт перехвачен
	bool turbo;		// турбо
	int getport(int);
	void out (int,uchar);
	uchar in (int);
};
BDI bdi;

struct Floppy {
	uchar id;		// 0..3 = A..D
	bool insert;		// disk inserted
	bool side;		// lower/upper
	bool protect;		// write protected
	bool trk80;		// drive have 80 tracks
	bool dblsid;		// drive have 2 heads (DS)
	bool index;		// index pulse
	int speed;		// 42000 - 6ms
	int count;		// delay counter (ticks). if =0, drive is ready
	int scount;		// delay for stepping
	uchar iback;		// countdown for index (100);
	int spin;		// spin counter (224 ticks = 1 byte)
	uchar trk,rtrk;		// physic, logic track number
	int pos;		// 0..TRACKLEN-1	current byte position
	uchar data[170][TRACKLEN];	// disk data (full)
	bool changed;		// disk changed
	uchar field,fnext;	// текущее и следующее поле (0:none; 1:address; 2:data-FB; 3:data-F8
	int datalen;			// оставшаяся длина поля
	uchar bufer[6];		// кэширование заголовка сектора
	std::string path;		// путь к файлу
	void tick();
	void step(bool);
	void format();
	void formtrack(uchar,uchar*);
	void load(std::string,uchar);
	bool getsector(uchar,uchar,uchar*);
	void save(std::string,uchar);
	bool savecha();
	bool eject();
};
Floppy flop[4];

struct VG93 {
	uchar com,cop,lop,fld;
	uchar drv,trk,sec,data;
	Floppy* fptr;
	bool block,mfm,irq,drq,sdir;
	bool mr;			// master reset
	int count;			// задержка;
	uchar mode,icount,scount;
	uchar wnum,wpos;	// номер и позиция задания
	uchar cpos,ccnt;	// позиция начала и счетчик цикла задачи
	bool hld;			// true = "wanna load head" (answer is floppy::head = true)
	bool seekerror,notfound,lostdata;
	void tick();
	void command(uchar);
	uchar getflag();
	void setmr(bool);
};
VG93 vg93;
